@@ -47,6 +47,14 @@
 #define FLAGS_USERANDOM		0x01
 #define FLAGS_SUBTRACTEXCLUDE	0x02
 
+struct socks_host {
+	TAILQ_ENTRY(socks_host) next;
+	struct addr host;
+	uint16_t port;
+};
+
+TAILQ_HEAD(socksq, socks_host);
+
 struct argument;
 
 struct address_slot {
@@ -122,7 +130,7 @@ int ipv4toa(char *, size_t, void *);
 void waitforcommands(int, int);
 
 void argument_free(struct argument *);
-void postres(struct argument *, char *);
+void postres(struct argument *, const char *fmt, ...);
 void printres(struct argument *, uint16_t, char *);
 
 int probe_haswork(void);
